(***********************************************************************)
(*  v      *   The Coq Proof Assistant  /  The Coq Development Team    *)
(* <O___,, *        INRIA-Rocquencourt  &  LRI-CNRS-Orsay              *)
(*   \VV/  *************************************************************)
(*    //   *      This file is distributed under the terms of the      *)
(*         *       GNU Lesser General Public License Version 2.1       *)
(***********************************************************************)

(*i $Id: typeops.mli,v 1.41 2001/11/12 12:37:59 barras Exp $ i*)

(*i*)
open Names
open Univ
open Term
open Environ
(*i*)

(*s Typing functions (not yet tagged as safe) *)

val infer      : env -> constr       -> unsafe_judgment * constraints
val infer_v    : env -> constr array -> unsafe_judgment array * constraints
val infer_type : env -> types        -> unsafe_type_judgment * constraints

type local_entry =
  | LocalDef of constr
  | LocalAssum of constr

val infer_local_decls :
  env -> (identifier * local_entry) list
    -> env * Sign.rel_context * constraints

(*s Basic operations of the typing machine. *)

(* If [j] is the judgement $c:t$, then [assumption_of_judgement env j]
   returns the type $c$, checking that $t$ is a sort. *)

val assumption_of_judgment :  env -> unsafe_judgment -> types
val type_judgment          :  env -> unsafe_judgment -> unsafe_type_judgment

(*s Type of sorts. *)
val judge_of_prop_contents : contents -> unsafe_judgment
val judge_of_type          : universe -> unsafe_judgment

(*s Type of a bound variable. *)
val judge_of_relative : env -> int -> unsafe_judgment

(*s Type of variables *)
val judge_of_variable : env -> variable -> unsafe_judgment

(*s type of a constant *)
val judge_of_constant : env -> constant -> unsafe_judgment

(*s Type of application. *)
val judge_of_apply : 
  env -> unsafe_judgment -> unsafe_judgment array
    -> unsafe_judgment * constraints

(*s Type of an abstraction. *)
val judge_of_abstraction : 
  env -> name -> unsafe_type_judgment -> unsafe_judgment 
    -> unsafe_judgment

(*s Type of a product. *)
val judge_of_product :
  env -> name -> unsafe_type_judgment -> unsafe_type_judgment 
    -> unsafe_judgment

(* s Type of a let in. *)
val judge_of_letin :
  env -> name -> unsafe_judgment -> unsafe_type_judgment -> unsafe_judgment 
    -> unsafe_judgment * constraints

(*s Type of a cast. *)
val judge_of_cast :
  env -> unsafe_judgment -> unsafe_type_judgment
    -> unsafe_judgment * constraints

(*s Inductive types. *)

val judge_of_inductive : env -> inductive -> unsafe_judgment

val judge_of_constructor : env -> constructor -> unsafe_judgment

(*s Type of Cases. *)
val judge_of_case : env -> case_info
  -> unsafe_judgment -> unsafe_judgment -> unsafe_judgment array
    -> unsafe_judgment * constraints

(* Typecheck general fixpoint (not checking guard conditions) *)
val type_fixpoint : env -> name array -> types array 
    -> unsafe_judgment array -> constraints
